from typing import Tuple

from sqlalchemy import select, delete, or_
from sqlalchemy import ScalarResult, Result
from sqlalchemy.orm.attributes import flag_modified

from ..database.db import async_session
from ..database.models import Contact, Apply, UserAccount, UserProfile


async def insert_contact_friend(
    relation: int,
    applicant: str,
    recipient: str,
    group_chat_id: str | None,
    applicant_setting: dict,
    recipient_setting: dict,
):
    session = async_session()
    try:
        await session.execute(
            delete(Apply).where(
                Apply.recipient == recipient,
                Apply.applicant == applicant,
                Apply.relation == relation,
            )
        )
        res: ScalarResult[Contact] = await session.scalars(
            select(Contact).where(
                or_(Contact.user_id == applicant, Contact.user_id == recipient)
            )
        )
        for contact in res.all():
            if contact.user_id == recipient:
                contact.friends[applicant] = recipient_setting
                flag_modified(contact, "friends")
            else:
                contact.friends[recipient] = applicant_setting
                flag_modified(contact, "friends")

        session.add_all(res.all())
        await session.commit()
    except Exception:
        raise Exception
    finally:
        await session.close()


async def select_contact_all(user_id: str) -> Contact:
    session = async_session()
    res: ScalarResult[Contact] = await session.scalars(
        select(Contact).where(Contact.user_id == user_id)
    )
    await session.close()
    return res.one()


async def select_friends_group_chats(
    friend_ids: list[str],
) -> list[Tuple[UserAccount, UserProfile]]:
    session = async_session()
    res: Result[list[Tuple[UserAccount, UserProfile]]] = await session.execute(
        select(UserAccount, UserProfile)
        .join(UserAccount.profile)
        .where(UserAccount.id.in_(friend_ids))
    )

    return res.all()


async def update_friend_setting(
    user_id: str,
    friend_id: str,
    remark: str | None,
    group: str | None,
) -> Contact:
    session = async_session()
    res = await session.scalars(select(Contact).where(Contact.user_id == user_id))
    contact: Contact = res.one()
    if remark:
        contact.friends[friend_id]["friendRemark"] = remark
    else:
        contact.friends[friend_id]["friendGroup"] = group
    flag_modified(contact, "friends")
    session.add(contact)
    await session.commit()
    await session.close()

    return contact


async def update_groups(
    user_id: str,
    groups: list[str],
    group_name_change_pair: list[list[str]],
    deleted_origin_groups: list[str],
    default_group: str,
) -> Contact:
    session = async_session()
    res = await session.scalars(select(Contact).where(Contact.user_id == user_id))
    contact = res.one()
    contact.friend_groups = groups
    contact.default_group = default_group

    for pair, deleted_group in zip(group_name_change_pair, deleted_origin_groups):
        if pair[1] == "" and deleted_group == "":
            continue
        for friend_id, friend_setting in contact.friends.items():
            if pair[0] == friend_setting["friendGroup"]:
                contact.friends[friend_id]["friendGroup"] = pair[1]
            if friend_setting["friendGroup"] == deleted_group:
                contact.friends[friend_id]["friendGroup"] = default_group

    flag_modified(contact, "friends")
    session.add(contact)
    await session.commit()
    await session.close()

    return contact


async def delete_friend(user_id: str, friend_id: str) -> Contact:
    session = async_session()
    res1 = await session.scalars(select(Contact).where(Contact.user_id == user_id))
    res2 = await session.scalars(select(Contact).where(Contact.user_id == friend_id))

    my_contact = res1.one()
    friend_contact = res2.one()

    del my_contact.friends[friend_id]
    del friend_contact.friends[user_id]

    flag_modified(my_contact, "friends")
    flag_modified(friend_contact, "friends")

    session.add_all([my_contact, friend_contact])
    await session.commit()
    await session.close()

    return my_contact
