from fastapi import APIRouter, Query
from fastapi.responses import FileResponse
from pydantic import BaseModel

from src.crud import apply_crud, user_crud, contact_crud
from ..response_models.apply_response import (
    BaseResponseModel,
    ApplyListResponse,
    ApplicantProfilesResponse,
)
from ..utils.static_file import create_zip_file

router = APIRouter(prefix="/apply", tags=["apply"])


class ApplyInfo(BaseModel):
    relation: int
    applicant: str
    recipient: str
    group_chat_id: str | None = None
    hello: str
    setting: dict[str, str] = {}


class AcceptInfo(BaseModel):
    relation: int
    applicant: str
    recipient: str
    group_chat_id: str | None = None
    applicant_setting: dict
    recipient_setting: dict


class RefuseInfo(BaseModel):
    relation: int
    applicant: str
    recipient: str


@router.post("/friend", response_model=BaseResponseModel)
async def apply_friend(apply_info: ApplyInfo):
    await apply_crud.insert_apply(**apply_info.model_dump())
    return {"code": 10600, "msg": "Apply Friend Successfully"}


@router.get("/list", response_model=ApplyListResponse)
async def get_apply_list(recipient: str):
    res = await apply_crud.select_apply_all(recipient)
    if not res:
        return {"code": 10601, "msg": "The Apply List is Empty"}
    else:
        data = []
        for apply in res:
            data.append(apply.to_dict())
        print(data)
        return {
            "code": 10600,
            "msg": "Get All Apply List Successfully",
            "data": data,
        }


@router.get("/applicant_profiles", response_model=ApplicantProfilesResponse)
async def get_applicant_profiles(applicant_ids: list[str] = Query(default=None)):
    res = await user_crud.select_multiuser_info(applicant_ids)
    applicant_profiles = {}
    for applicant in res:
        applicant_profiles[applicant[0].id] = applicant[0].to_dict()
        applicant_profiles[applicant[0].id].update(applicant[1].to_dict())
    print(applicant_profiles)
    return {
        "code": 10600,
        "msg": "Get Applicant Information Successfully",
        "data": applicant_profiles,
    }


@router.get("/applicant_avatars")
async def download_applicant_avatars(avatars: list[str] = Query(default=None)):
    file_path = create_zip_file(avatars, "avatars")
    return FileResponse(file_path)


@router.post("/accept", response_model=BaseResponseModel)
async def accept_apply(accept_info: AcceptInfo):
    try:
        await contact_crud.insert_contact_friend(**accept_info.model_dump())
        return {"code": 10600, "msg": "Add Friend Successfully"}
    except Exception as e:
        print(f"接受添加好友请求出错....: {e}")
        return {"code": 10601, "msg": "Something Went Wrong On the Server"}


@router.post("/refuse", response_model=BaseResponseModel)
async def refuse_apply(refuse_info: RefuseInfo):
    await apply_crud.delete_apply(**refuse_info.model_dump())
    return {"code": 10600, "msg": "Refuse Apply Successfully"}
