from fastapi import APIRouter
from pydantic import BaseModel

from ..crud import contact_crud, user_crud
from ..response_models.contact_response import (
    BaseResponseModel,
    ContactResponse,
    ContactAccountProfileResponse,
)

router = APIRouter(prefix="/contact", tags=["contact"])


class ContactIds(BaseModel):
    friend_ids: list[str]
    group_chat_ids: list[str] | None = None


class ChangeFriendSetting(BaseModel):
    user_id: str
    friend_id: str
    remark: str | None = None
    group: str | None = None


class ManageGroups(BaseModel):
    user_id: str
    groups: list[str]
    group_name_change_pair: list[list[str]]
    deleted_origin_groups: list[str]
    default_group: str


class MyselfFriendId(BaseModel):
    user_id: str
    friend_id: str


@router.get("", response_model=ContactResponse)
async def get_contact(id: str):
    contact = await contact_crud.select_contact_all(id)
    print(contact.to_dict())
    return {"code": 10700, "msg": "Get Contact Successfully", "data": contact.to_dict()}


@router.post("/profiles", response_model=ContactAccountProfileResponse)
async def get_contact_account_profiles(contact_ids: ContactIds):
    res = await contact_crud.select_friends_group_chats(contact_ids.friend_ids)

    friends_account_profiles = {}

    for account, profile in res:
        friends_account_profiles[account.id] = account.to_dict()
        friends_account_profiles[account.id].update(profile.to_dict())

    return {
        "code": 10700,
        "msg": "Get Contact Profiles Successfully",
        "data": {"friends": friends_account_profiles},
    }


@router.post("/change/friend_setting", response_model=ContactResponse)
async def change_friend_setting(friend_remark: ChangeFriendSetting):
    res = await contact_crud.update_friend_setting(**friend_remark.model_dump())
    return {
        "code": 10700,
        "msg": "change Friend Setting Successfully",
        "data": res.to_dict(),
    }


@router.post("/manage_groups", response_model=ContactResponse)
async def manage_groups(group_info: ManageGroups):
    res = await contact_crud.update_groups(**group_info.model_dump())
    return {"code": 10700, "msg": "Manage Groups Successfully", "data": res.to_dict()}


@router.post("/delete/friend", response_model=ContactResponse)
async def delete_friend(myself_friend_id: MyselfFriendId):
    res = await contact_crud.delete_friend(**myself_friend_id.model_dump())
    return {"code": 10700, "msg": "Delete Friend Successfully", "data": res.to_dict()}
