from fastapi import APIRouter
from fastapi import Depends
from fastapi.security import OAuth2PasswordRequestForm
from pydantic import BaseModel

from jose import ExpiredSignatureError, JWTError

from ..crud.user_crud import select_account_by
from ..utils.password import verify_password
from ..utils.token_handler import (
    create_signin_token,
    oauth2_scheme,
    verify_signin_token,
)
from ..response_models.user_response import (
    UserAccountResponse,
    TokenCreationResponse,
    TokenSigninResponse,
)

router = APIRouter(prefix="/signin", tags=["signin"])


class TokenPayload(BaseModel):
    id: str
    device_id: str


@router.post("/username", response_model=UserAccountResponse)
async def signin_by_username(form_data: OAuth2PasswordRequestForm = Depends()):
    username = form_data.username
    password = form_data.password
    is_existence, user = await select_account_by("username", username)

    if not is_existence:
        return {"code": 10201, "msg": "Username or Password Is Incorrect"}

    is_correct = verify_password(password, user.password)

    if is_correct:
        return {"code": 10200, "msg": "Sign in Successfully", "data": user.to_dict()}
    else:
        return {"code": 10201, "msg": "Username or Password Is Incorrect"}


@router.post("/token", response_model=TokenCreationResponse)
async def create_token(token_payload: TokenPayload):
    token = create_signin_token(**token_payload.model_dump())
    return {"code": 10200, "msg": "Create Token Successfully", "token": token}


@router.get("/token", response_model=TokenSigninResponse)
async def signin_by_token(token: str = Depends(oauth2_scheme)):
    try:
        new_token, id = verify_signin_token(token)
        _, user = await select_account_by("id", id)
        if new_token:
            return {
                "code": 10200,
                "msg": "Sign in Successfully",
                "data": user.to_dict(),
                "token": new_token,
            }
        else:
            return {
                "code": 10200,
                "msg": "Sign in Successfully",
                "data": user.to_dict(),
                "token": token,
            }
    except ExpiredSignatureError:
        return {
            "code": 9999,
            "msg": "Token has Expired",
        }
    except JWTError:
        return {
            "code": 9998,
            "msg": "Token Is Not Right",
        }
