from fastapi import APIRouter, BackgroundTasks
from pydantic import BaseModel

from ..crud import user_crud
from ..utils import password
from ..utils.email_code import send_email, has_code, verify_code
from ..response_models.base import BaseResponseModel

router = APIRouter(prefix='/user_account', tags=['user_account'])


class ChangedAccount(BaseModel):
    id: str
    username: str | None = None
    email: str | None = None
    password: str | None = None
    code: str | None = None


@router.post('/change/username', response_model=BaseResponseModel)
async def change_username(changed_account: ChangedAccount):
    is_existed, user = await user_crud.select_account_by('username', changed_account.username)
    if is_existed:
        return {'code': 10401, 'msg': f'This Username ({changed_account.username}) Has Been Used'}

    await user_crud.update_account('username', changed_account.id, changed_account.username)

    return {'code': 10400, 'msg': 'Update Username Successfully'}


@router.get('/get/email_code', response_model=BaseResponseModel)
async def get_change_email_code(email: str, background_tasks: BackgroundTasks):
    is_existed, _ = await user_crud.select_account_by('email', email)
    if is_existed:
        return {'code': 10401, 'msg': f'This Email ({email}) Has Been Used'}

    if has_code(email):
        return {'code': 10402, 'msg': f'Code of Email ({email}) Is Still Available'}

    background_tasks.add_task(send_email, email)
    return {'code': 10400, 'msg': 'Send Verification Code Successfully'}


@router.post('/change/email', response_model=BaseResponseModel)
async def change_email(changed_account: ChangedAccount):
    is_correct = verify_code(changed_account.email, changed_account.code)
    if not is_correct:
        return {'code': 10403, 'msg': f'Email Code ({changed_account.code}) Is Not Correct'}

    await user_crud.update_account('email', changed_account.id, changed_account.email)

    return {'code': 10400, 'msg': 'Update Email Successfully'}


@router.get('/get/password_code', response_model=BaseResponseModel)
async def get_change_password_code(email: str, background_tasks: BackgroundTasks):
    if has_code(email):
        return {'code': 10402, 'msg': f'Code of Email ({email}) Is Still Available'}

    background_tasks.add_task(send_email, email)
    return {'code': 10400, 'msg': 'Send Verification Code Successfully'}


@router.post('/change/password', response_model=BaseResponseModel)
async def change_password(changed_account: ChangedAccount):
    is_correct = verify_code(changed_account.email, changed_account.code)
    if not is_correct:
        return {'code': 10403, 'msg': f'Email Code ({changed_account.code}) Is Not Correct'}

    hashed_password = password.get_hashed_password(changed_account.password)

    await user_crud.update_account('password', changed_account.id, hashed_password)

    return {'code': 10400, 'msg': 'Update Email Successfully'}