from typing import Optional

from fastapi import APIRouter
from pydantic import BaseModel
from fastapi.responses import FileResponse, JSONResponse
from fastapi.encoders import jsonable_encoder

from anyio import open_file

from ..crud import user_crud
from ..response_models.user_response import (
    BaseResponseModel,
    UserProfileResponse,
    UserAvatarResponse,
)
from ..utils import static_file


router = APIRouter(prefix="/user_profile", tags=["user_profile"])


class Uint8List(BaseModel):
    file: list


class ChangedProfile(BaseModel):
    id: str
    nickname: Optional[str] = None
    location: Optional[str] = None
    birthday: Optional[str] = None
    gender: Optional[str] = None
    sign: Optional[str] = None


@router.get("/my", response_model=UserProfileResponse)
async def get_profile(id: str):
    profile = await user_crud.select_profile(id)
    return JSONResponse(
        content=jsonable_encoder(
            {
                "code": 10300,
                "msg": "Get My Profile Successfully",
                "data": profile.to_dict(),
            },
        )
    )


@router.get("/avatar")
async def download_avatar(avatar_filename: str):
    avatar_dir_path = static_file.create_dir("avatars")
    return FileResponse(avatar_dir_path / avatar_filename)


@router.post("/change/avatar", response_model=UserAvatarResponse)
async def change_avatar(id: str, file: Uint8List):
    avatar_dir_path = static_file.create_dir("avatars")
    avatar_filename = static_file.create_avatar_filename()

    async with await open_file(avatar_dir_path / avatar_filename, "wb") as f:
        await f.write(bytearray(file.file))

    await user_crud.update_profile_avatar(id, avatar_filename)

    return {"code": 10300, "msg": "Update Avatar Successfully", "data": avatar_filename}


@router.post("/change/{aspect}", response_model=BaseResponseModel)
async def change_profile(aspect: str, changed_profile: ChangedProfile):
    match aspect:
        case "basic":
            await user_crud.update_profile_basic(
                changed_profile.id,
                changed_profile.nickname,
                changed_profile.location,
                changed_profile.birthday,
                changed_profile.gender,
            )
        case "sign":
            await user_crud.update_profile_sign(
                changed_profile.id, changed_profile.sign
            )
        case _:
            return {"code": 10301, "msg": f"No /change/{aspect} Path"}
    return {"code": 10300, "msg": f"Update {aspect} Profile Successfully"}
